USE RealSecureDB
go

IF OBJECT_ID('iss_Document_Add') IS NOT NULL DROP PROCEDURE dbo.iss_Document_Add
go
/*
This procedure is used by the iss_installer to seed documents during a DBSP
The installer can seed support files or named policies

A named policy should be seeded in the default repository but not deployed
A support file should be added to the binary data table only
exec iss_document_add 'GlobalObject', '

select * from deployedpolicy where namespaceid = 126

*/
CREATE PROCEDURE dbo.iss_Document_Add
	@DocumentType		VARCHAR(20), --Policy or BinaryData or GlobalObject
	@FileName		NVARCHAR(200), --BinaryData file name or Policy name
	@Namespace 		VARCHAR(255),
	@Version		VARCHAR(100) = null,
	@Value			IMAGE,
	@CheckSum		INT,
	@OverwriteDoc		TINYINT = 1,
	@BinaryDataType		TINYINT = 1,
	@LastModifiedByPrincipalID INT = NULL,
	@MIMEType		VARCHAR(50) = NULL,
	@Secured		TINYINT = 0,
	@BinaryDataID		INT = NULL	OUTPUT,
	@LastModifiedAt 	DATETIME = NULL	OUTPUT,
	@Deploy			TINYINT = 0
AS 
/* IBM Confidential - OCO Source Materials */
/* Copyright (c) IBM Corp. 1992, 2008 */
/* The source code for this program is not published or otherwise divested of its trade secrets, */
/* irrespective of what has been deposited with the U.S. Copyright Office. */ 
begin try

DECLARE @SaveRowCnt int, @ProcName sysname 
SELECT @ProcName = OBJECT_NAME(@@ProcID)
SET NOCOUNT ON

--default behavior is to overwrite document if not specified	
SELECT @OverwriteDoc = ISNULL(@OverwriteDoc,1)
--default binarydatatype if null
SELECT @BinaryDataType = ISNULL(@BinaryDataType,1) 
--default name
SELECT @FileName = ISNULL(@FileName,'Default') 

DECLARE	@NamespaceID	INT, 
	@RepositoryID INT,
	@PolicyRepositoryID INT,
	@PolicyRepositoryVersionID INT,
	@MaxVersion INT,
	@DefaultGroupID INT

--Add Namespace and/or get NamespaceID
EXEC iss_AddNamespace @Namespace = @Namespace, @NamespaceID = @NamespaceID OUTPUT

-- Grab default RepositoryID
SELECT @DefaultGroupID = GroupID 
FROM Groups WITH (NOLOCK) 
WHERE Deleted = 0 
AND ParentGroupID IS NULL

SELECT @RepositoryID = RepositoryID 
FROM Repository WITH (NOLOCK)
WHERE GroupID = @DefaultGroupID

BEGIN TRAN
IF @DocumentType = 'BinaryData'
	BEGIN
	SELECT TOP 1 @BinaryDataID = BinaryDataID
	FROM BinaryData WITH (UPDLOCK)
	WHERE FileName = @FileName 
	AND BinaryDataType = @BinaryDataType
	AND BinaryDataID NOT IN (SELECT BinaryDataID FROM PolicyRepositoryVersion WITH (NOLOCK))
	AND BinaryDataID NOT IN (SELECT BinaryDataID FROM PolicyDocumentCollection WITH (NOLOCK))

	IF @BinaryDataID IS NULL
		BEGIN
		EXEC iss_AddBinaryData 
				@BinaryDataID = @BinaryDataID	OUTPUT,
				@BinaryDataType = @BinaryDataType, 
				@Value = @Value,
				@CheckSum = @CheckSum,
				@FileName = @FileName,
				@LastModifiedByPrincipalID = NULL,
				@MIMEType = NULL,
				@Secured = 0
		END
	ELSE IF @OverwriteDoc = 1 AND (@CheckSum <> (SELECT CheckSum FROM BinaryData WITH (UPDLOCK) WHERE BinaryDataID = @BinaryDataID))
		BEGIN
		EXEC iss_UpdateBinaryDataValue 
				@BinaryDataID = @BinaryDataID,
				@Value = @Value,
				@CheckSum = @CheckSum,
				@LastModifiedByPrincipalID = NULL,
				@MIMEType = NULL,
				@Secured = 0
		END 
	END
ELSE 
	BEGIN
	--Check for existence of document
	SELECT	TOP 1 @PolicyRepositoryID = prv.PolicyRepositoryID, 
		@BinaryDataID = prv.BinaryDataID,
		@MaxVersion = prv.Version
	FROM	PolicyRepositoryVersion prv WITH (UPDLOCK)
		INNER JOIN PolicyRepository pr WITH (UPDLOCK)
		ON prv.PolicyRepositoryID = pr.PolicyRepositoryID
	WHERE 	pr.NamespaceID = @NamespaceID 
	AND	pr.SchemaVersion = @Version
	AND	(pr.Name = @FileName OR @DocumentType = 'GlobalObject')
	AND	pr.RepositoryID = @RepositoryID
	ORDER BY prv.Version DESC

	SELECT	@SaveRowCnt = @@ROWCOUNT

	IF @SaveRowCnt = 1 AND 
		((@CheckSum = (SELECT CheckSum FROM BinaryData WITH (NOLOCK) WHERE BinaryDataID = @BinaryDataID))
			OR @OverwriteDoc = 0)
		--checksums match, drop out, nothing to do, policy is the same as what is already stored
		--or we've been told to not overwrite the existing document
		BEGIN
		COMMIT TRAN
		RETURN(0)
		END

	IF @SaveRowCnt = 1 -- exists in the repository (and checksum didn't match, so content is different)
		BEGIN
		IF @DocumentType = 'GlobalObject' AND @OverwriteDoc = 1  -- just overwrite the content of global objects
			BEGIN
			EXEC iss_UpdateBinaryDataValue 
					@BinaryDataID = @BinaryDataID,
					@Value = @Value,
					@CheckSum = @CheckSum,
					@LastModifiedByPrincipalID = NULL,
					@MIMEType = NULL,
					@Secured = 0
			END
		ELSE
			BEGIN
			-- The document already exists, so insert a new BinaryData and PolicyRepositoryVersion 
			--Add Binary Data record
			SELECT @Secured = ISNULL(@Secured, 0)
			EXEC iss_AddBinaryData 
					@BinaryDataID = @BinaryDataID	OUTPUT,
					@BinaryDataType = @BinaryDataType,
					@Value = @Value,
					@CheckSum = @CheckSum,
					@FileName = @Namespace, --use Namespace for now
					@LastModifiedByPrincipalID = @LastModifiedByPrincipalID,
					@MIMEType = @MIMEType,
					@Secured = @Secured

			--Get highest version number for given policyrepositoryid
			INSERT INTO PolicyRepositoryVersion(Version, BinaryDataID, PolicyRepositoryID, LastModifiedByPrincipalID, LastModifiedAt, DeploymentCount)
			SELECT ISNULL(@MaxVersion,0) + 1, @BinaryDataID, @PolicyRepositoryID, @LastModifiedByPrincipalID, GETUTCDATE(),ISNULL(@Deploy,0)

			SELECT @PolicyRepositoryVersionID = @@IDENTITY
			IF @Deploy = 1
				BEGIN
				UPDATE PolicyRepositoryVersion
				SET DeploymentCount = 0
				WHERE PolicyRepositoryID = @PolicyRepositoryID
				AND PolicyRepositoryVersionID != @PolicyRepositoryVersionID

				IF EXISTS (SELECT 1 FROM DeployedPolicy WITH (UPDLOCK) WHERE GroupID = @DefaultGroupID AND NamespaceID = @NamespaceID AND SchemaVersion = @Version)
					BEGIN
					UPDATE DeployedPolicy
					SET	DeployTime = GETUTCDATE(),
						PolicyRepositoryVersionID = @PolicyRepositoryVersionID,
						DeployedByPrincipalID = @LastModifiedByPrincipalID
					WHERE GroupID = @DefaultGroupID 
					AND NamespaceID = @NamespaceID 
					AND SchemaVersion = @Version
					END
				ELSE
					BEGIN
					INSERT INTO DeployedPolicy (GroupID, DeployTime, DeployedByPrincipalID, PolicyRepositoryVersionID, NamespaceID, SchemaVersion)
					SELECT @DefaultGroupID, GETUTCDATE(), @LastModifiedByPrincipalID, @PolicyRepositoryVersionID, @NamespaceID, @Version
					END
				END 
			END -- don't overwrite non-global objects
		END -- of the document already exists in the repository
	ELSE
		BEGIN
		-- New document, so insert BinaryData, PolicyRepository, PolicyRepositoryVersion, DeployedPolicy
		--Add Binary Data record		
		SELECT @Secured = ISNULL(@Secured, 0)
		EXEC iss_AddBinaryData 
				@BinaryDataID = @BinaryDataID	OUTPUT,
				@BinaryDataType = @BinaryDataType, 
				@Value = @Value,
				@CheckSum = @CheckSum,
				@FileName = @Namespace, --use Namespace for now
				@LastModifiedByPrincipalID = @LastModifiedByPrincipalID,
				@MIMEType = @MIMEType,
				@Secured = @Secured

		IF NOT EXISTS (SELECT 1 FROM PolicyRepository WITH (UPDLOCK) WHERE [Name] = @FileName AND NamespaceID = @NamespaceID
			AND SchemaVersion = @Version AND RepositoryID = @RepositoryID)
			BEGIN
			INSERT INTO PolicyRepository
				([Name], NamespaceID, SchemaVersion, LastModifiedAt, RepositoryID)
			VALUES
				(@FileName, @NamespaceID, @Version, GETUTCDATE(), @RepositoryID)
	
			SELECT	@PolicyRepositoryID = @@IDENTITY
			END

		INSERT INTO PolicyRepositoryVersion
			(Version, BinaryDataID, PolicyRepositoryID, LastModifiedByPrincipalID, LastModifiedAt, DeploymentCount)
		SELECT 1, @BinaryDataID, @PolicyRepositoryID, @LastModifiedByPrincipalID, GETUTCDATE(), ISNULL(@Deploy,0)
		
		SELECT @PolicyRepositoryVersionID = @@IDENTITY

		IF @Deploy = 1
			BEGIN
			UPDATE PolicyRepositoryVersion
			SET DeploymentCount = 0
			WHERE PolicyRepositoryID = @PolicyRepositoryID
			AND PolicyRepositoryVersionID != @PolicyRepositoryVersionID

			IF EXISTS (SELECT 1 FROM DeployedPolicy WITH (UPDLOCK) WHERE GroupID = @DefaultGroupID AND NamespaceID = @NamespaceID AND SchemaVersion = @Version)
				BEGIN
				UPDATE DeployedPolicy
				SET	DeployTime = GETUTCDATE(),
					PolicyRepositoryVersionID = @PolicyRepositoryVersionID,
					DeployedByPrincipalID = @LastModifiedByPrincipalID
				WHERE GroupID = @DefaultGroupID 
				AND NamespaceID = @NamespaceID 
				AND SchemaVersion = @Version
				END
			ELSE
				BEGIN
				INSERT INTO DeployedPolicy (GroupID, DeployTime, DeployedByPrincipalID, PolicyRepositoryVersionID, NamespaceID, SchemaVersion)
				SELECT @DefaultGroupID, GETUTCDATE(), @LastModifiedByPrincipalID, @PolicyRepositoryVersionID, @NamespaceID, @Version
				END
			END 
		END
	END

COMMIT TRAN
RETURN (0)
end try

begin catch
	exec iss_error_catch
end catch
GO

GRANT EXECUTE ON iss_Document_Add to IssApplication
GO

